package NGCP::Service::Meta;

use strict;
use warnings;
use feature qw(state);

use Exporter qw(import);
use List::Util qw(any none);
use YAML::XS;

our $VERSION = '0.01';
our @EXPORT = qw(
    host_meta
    host_get_ngcp_type
    host_get_init_system
    host_get_process_handler
    host_get_roles
    host_has_role

    serv_get_config_name
    serv_meta
    serv_get_desc_byname
    serv_in_ngcp_type
    serv_in_role
    serv_in_group
    serv_is_managed
);

# Support testing env.
my $NGCP_INC = $ENV{NGCP_INC} // '/etc/ngcp-service/nsservices.yml';
my $NGCP_CONFIG = $ENV{NGCP_CONFIG} // '/etc/ngcp-service/nsconfig.yml';

sub host_meta
{
    state $conf = YAML::XS::LoadFile($NGCP_CONFIG);

    return $conf->{host};
}

sub host_get_ngcp_type
{
    return host_meta()->{ngcp_type};
}

sub host_get_init_system
{
    return host_meta()->{init_system};
}

sub host_get_process_handler
{
    return host_meta()->{process_handler}
}

sub host_get_roles
{
    return host_meta()->{roles};
}

sub host_has_role
{
    my $role = shift;

    return 1 if $role eq 'all';
    return any { $_ eq $role } @{host_get_roles()};
}


sub serv_get_config_name
{
    return $NGCP_INC;
}

sub serv_meta
{
    state $serv = YAML::XS::LoadFile($NGCP_INC);

    return $serv->{services};
}

sub _serv_meta_gen_aliases
{
    my $meta = serv_meta();
    my $aliases;

    foreach my $service (keys %{$meta}) {
        my @aliases;

        if (exists $meta->{$service}->{aliases}) {
            push @aliases, @{$meta->{$service}->{aliases}};
        }

        foreach my $prog (qw(systemd sysv monit supervisor)) {
            my $alias = $meta->{$service}->{$prog};
            push @aliases, $alias if defined $alias;
        }
        foreach my $alias (@aliases) {
            $aliases->{$alias} = $meta->{$service};
            $aliases->{$alias}->{name} //= $service;
        }
    }

    return $aliases;
}

sub serv_meta_aliases
{
    state $aliases = _serv_meta_gen_aliases();

    return $aliases;
}

# Get service definitions with default properties added.
sub serv_get_desc_byname
{
    my ($service) = shift;

    my $servdesc = serv_meta()->{$service} // serv_meta_aliases()->{$service};

    return unless defined $servdesc;

    $servdesc->{name} //= $service;
    $servdesc->{group} //= [ ];
    $servdesc->{node} //= 'all';
    $servdesc->{role} //= [ 'all' ];
    $servdesc->{enable} //= 'yes';
    $servdesc->{ngcp_type} //= [ 'spce', 'sppro', 'carrier' ];

    return $servdesc;
}

sub serv_in_ngcp_type
{
    my $servdesc = shift;
    my $host_type = host_get_ngcp_type();

    return any { $_ eq $host_type } @{$servdesc->{ngcp_type}};
}

sub serv_in_role
{
    my $servdesc = shift;

    return 1 if any { $_ eq 'all' } @{$servdesc->{role}};

    foreach my $role (@{$servdesc->{role}}) {
        return 1 if host_has_role($role);
    }

    return 0;
}

sub serv_in_group
{
    my ($servdesc, $match_group) = @_;

    return 1 unless defined $match_group;

    foreach my $group (@{$servdesc->{group}}) {
        return 1 if $group eq $match_group;
    }

    return 0;
}

sub serv_is_managed
{
    my $servdesc = shift;

    # If we are on a CE, the service is never managed.
    return 0 if host_get_ngcp_type() eq 'spce';

    # If the service should only be run on the active node, it means that
    # HA is managing it, and monit only monitors it.
    return 1 if exists $servdesc->{node} and $servdesc->{node} eq 'active';
    return 0;
}

1;
